
% the purposes of this m-file are to (1) create two 90 out of phase
% signals, such as might be generated as the outputs of an optical encoder
% attached to a moving shaft, (2) use these signals as input to
% a Stateflow model of quadrature decoding, (3) plot the resulting change
% in counter value (and thus decoded position) vs time

%
% We make several simplifying assumptions
%   1. Both quadrature signals have value zero initially
%   2. Transitions only occur at integer multiples
%      of a time unit T.
%   3. It is up to the user to specify out-of-phase signals
%      that do not have simultaneous transitions on both channels, 
%      as these are not physically possible

N = 25; % we do simulation for N units of time, starting at 0
T = 1;  % duriation of time interval   

tmax = N*T;    % simulation is over the interval (0,tmax)

% input1 and input2 define the two quadrature signals. These MUST
% be vectors of length N, where 1 indicates a high signal, and 0
% indicates a low signal. The simplifying assumption that the 
% initial value of both signals is 0 requires the 
% the first entry in both vectors to be 0.
 
input1 = [0 1 1 0 0 1 1 0 0 1 1 1 0 0 1 1 1 0 0 1 1 0 0 0 0];
input2 = [0 0 1 1 0 0 1 1 0 0 1 0 0 1 1 0 1 1 0 0 1 1 0 0 0];


% we will now create the quadrature signals as a function of time
% to plot them and see nice sharp corners we will create values
% for them at intervals of deltat, where deltat << T

deltat = .001;

% the times at which the signals are to be evaluated
times = 0:deltat:(tmax-deltat);

for i = 1:length(times)
    quadrature1(i) = input1(fix(times(i))+1);
    quadrature2(i) = input2(fix(times(i))+1);
end


% plot quadrature signals vs time
figure(1)
clf
subplot(2,1,1)
plot(times,quadrature1,'b-')
axis([0  tmax  -.5  1.5])
ylabel('Channel 1 (primary)')
grid
h = gca;
set(h,'YGrid','off')
set(h,'XTick',1:25)


subplot(2,1,2)
plot(times,quadrature2,'r-')
axis([0  tmax  -.5  1.5])
grid
h = gca;
set(h,'YGrid','off')
set(h,'XTick',1:25)
ylabel('Channel 2 (secondary)')
xlabel('time, T units')


% We will now input the quadrature signals we have created to a model of
% a quadrature decoder

fqd1 = [ times' quadrature1'];
fqd2 = [ times' quadrature2'];

% uncomment the following line to run fast quad decode model
%sim('quad_decode_fast',tmax); mode = 'fast';  

% uncomment the following line to run 4X quad decode model
 sim('quad_decode_4X', tmax); mode = '4X';   


% plot the counter value and the input signals

figure(2)
clf

subplot(3,1,1); plot(tout,yout(:,1),'g-')
ylabel('counter value');
title([num2str(mode),' mode'])

subplot(3,1,2); 
plot(tout,yout(:,2),'b-')

ylabel('Channel 1 (primary)')
ylim([-.25 1.25])

subplot(3,1,3); 
plot(tout,yout(:,3),'r-')
ylabel('Channel 2 (secondary)')
ylim([-.25 1.25])

xlabel('time, T units');



